let NOAB = window.NOAB || {};
(function()
{
	let map, markerIcon;
	
	async function list(website, partialUrl)
	{
		let response = await fetch(`https://${website}/_api/${partialUrl}`);
		if (!response.ok)
			throw new Error(`Failed to get values from via the API.`);
		
		return await response.json();
	};
	this.List = list;
	
	function showSection(name, iframeID)
	{
		setSectionVisibility(name, true, iframeID);
	};
	this.ShowSection = showSection;
	
	function hideSection(name, iframeID)
	{
		setSectionVisibility(name, false, iframeID);
	};
	this.HideSection = hideSection;
	
	function setSectionVisibility(name, show, iframeID)
	{
		onControl(`fieldset[aria-label="${name}"]`, c => section[0].style.display = show ? "" : "none", iframeID);
	}
	this.SetSectionVisibility = setSectionVisibility;
	
	function showColumn(id, iframeID)
	{
		setColumnVisibility(id, true, iframeID);
	}
	this.ShowColumn = showColumn;
	
	function hideColumn(id, iframeID)
	{
		setColumnVisibility(id, false, iframeID);
	}
	this.HideColumn = hideColumn;
	
	function setColumnVisibility(id, show, iframeID)
	{
		onControl(id, c =>
		{
			const row = c.closest("tr");
			if (row)
				row.style.display = show ? "" : "none";
		}, iframeID);
	}
	this.SetColumnVisibility = setColumnVisibility;
	
	function hideSubgridIfEmpty(id, iframeID)
	{
		$(`#${id}`).on("loaded", () =>
		{
			onControl(id, subgrid =>
			{
				const hide = x =>
				{
					const row = x.closest("tr");
					if (row)
						row.style.display = "none";
				};
				
				if ($(`#${id} table tbody`).length == 0)
					hide(subgrid);
				else 
					onControl(`${id} table tbody`, tbody => 
					{
						if (tbody.children.length == 0)
							hide(subgrid);
					}, iframeID);
			}, iframeID);
		});
	}
	this.HideSubgridIfEmpty = hideSubgridIfEmpty;
	
	function hookColumnOnChange(id, runImmediately, f, iframeID)
	{
		onControl(id, c =>
		{
			c.onchange = () => f(c);
			if (runImmediately)
				f(c);
		}, iframeID);
	}
	this.HookColumnOnChange = hookColumnOnChange;
	
	function hookYesNoColumnOnChange(columnLogicalName, runImmediately, f, iframeID)
	{
		hookColumnOnChange(`${columnLogicalName}_1`, runImmediately, f, iframeID);
		hookColumnOnChange(`${columnLogicalName}_0`, runImmediately, f, iframeID);
	}
	this.HookYesNoColumnOnChange = hookYesNoColumnOnChange;
	
	function makeLookupDependent(id, parentID, iframeID)
	{
		hookColumnOnChange(parentID, true, control => 
		{
			onControl(`${id}_name`, control => 
			{
				control.value = '';
				const button = control.parentElement.querySelector(`button.clearlookupfield`);
				button.style.display = "none";
			}, iframeID);
			onControl(id, control => control.value = '', iframeID);
			setLookupReadOnly(`${id}_name`, control.value == '', iframeID);
		}, iframeID);
	}
	this.MakeLookupDependent = makeLookupDependent;
	
	function setColumnLabel(id, newLabel, iframeID)
	{
		onControl(id, c => c.innerHTML = newLabel, iframeID);
	}
	this.SetColumnLabel = setColumnLabel;
	
	function appendColumnLabel(id, newLabel, iframeID)
	{
		onControl(id, c => c.innerHTML += newLabel, iframeID);
	}
	this.AppendColumnLabel = appendColumnLabel;
	
	function deleteElement(id, iframeID)
	{
		onControl(id, c => c.remove(), iframeID);
	}
	this.DeleteElement = deleteElement;
	
	function makeFileAttachmentReadOnly(id, iframeID)
	{
		onControl(id, c => c.parentElement.parentElement.remove(), iframeID);
		onControl(`${id}_delete_button`, c => c.parentElement.remove(), iframeID);
	}
	this.MakeFileAttachmentReadOnly = makeFileAttachmentReadOnly;
	
	function setLookupReadOnly(id, readOnly, iframeID)
	{
		onControl(id, x => 
		{
			x.disabled = readOnly;
			if (readOnly)
				x.classList.add("readonly");
			else
				x.classList.remove("readonly");
			x.parentElement.querySelectorAll(`button`).forEach(y => y.disabled = readOnly);
		}, iframeID);
	}
	this.SetLookupReadOnly = setLookupReadOnly;
	
	function setDatePickerReadOnly(id, readOnly, iframeID)
	{
		onControl(id, c => 
		{
			c.parentElement.querySelectorAll(`input`).forEach(x => 
			{
				x.disabled = readOnly;
				if (readOnly)
					x.classList.add("readonly");
				else
					x.classList.remove("readonly");
			});
			c.parentElement.querySelector(`span`).style.display = 'none';
			c.parentElement.querySelector(`div`).style.width = '100%';
		}, iframeID);
	}
	this.SetDatePickerReadOnly = setDatePickerReadOnly;
	
	function setRequired(id, required)
	{
		const labelControlID = `${id}_label`;
		if (required)
		{
			if (typeof (Page_Validators) == 'undefined')
				return;
		
			const labelControl = $(`#${labelControlID}`)[0];
			labelControl.parentElement.classList.add("required");
		 
			let newValidator = document.createElement('span');
			newValidator.style.display = "none";
			newValidator.id = "RequiredFieldValidator" + id;
			newValidator.controltovalidate = "casetypecode";
			newValidator.errormessage = `<a href='#${labelControlID}'>${labelControl.innerText} is een vereist veld.</a>`;
			newValidator.validationGroup = "";
			newValidator.initialvalue = "";
			newValidator.evaluationfunction = () =>
			{
				let value;
				if ($(`#${id}`)[0] instanceof HTMLButtonElement)
					value = $(`#${id}_file_name`)[0].innerText;
				else			
					value = $(`#${id}`).val();
				return !(value == null || value == "")
			}
		 
			Page_Validators.push(newValidator);
		 
			$(`a[href='#${labelControlID}']`).on("click", () => scrollToAndFocus(labelControlID, id));
		}
		else
		{
			for (let i=0; i<Page_Validators.length; i++)
			{
				let validator = Page_Validators[i];
				if (validator != null && validator != undefined)
					if (validator.id == "RequiredFieldValidator" + id) 
					{
						Page_Validators.splice(i, 1);
						i--;
					}
			}
 
			$(`#${labelControlID}`).parent().removeClass("required");
		}
	}
	this.SetRequired = setRequired;
	
	function setDisabled(id, disabled, iframeID)
	{
		onControl(id, control => disabled ? control.setAttribute(`disabled`, ``) : control.removeAttribute(`disabled`), iframeID);
	}
	this.SetDisabled = setDisabled;
	
	function onControl(id, fx, iframeID)
	{
		let controls;
		if (iframeID)
			controls = $(`#${id}`, $(`#${iframeID}`).contents());
		else
			controls = $(`#${id}`);
		if (controls != null && controls.length > 0)
			fx(controls[0]);
	}
	this.OnControl = onControl;
	
	function minimizeXml(xml)
	{
		let dom = parseXml(xml);
		removeWhitespace(dom);
		return serializeXml(dom);
	}

	function parseXml(txt)
	{
		let xmlDoc;

		if (window.DOMParser)
		{
			const parser = new DOMParser();
			xmlDoc = parser.parseFromString(txt, "text/xml");
		}
		else // Internet Explorer
		{
			xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
			xmlDoc.async = false;
			xmlDoc.loadXML(txt);
		}

		return xmlDoc;
	}

	function serializeXml(node)
	{
		try
		{
			// XMLSerializer exists in certain browsers
			var serializer = new XMLSerializer();
			return serializer.serializeToString(node);
		}
		catch (e)
		{
			// Internet Explorer has a different approach to serializing XML
			return elem.xml;
		}

	}

	function removeWhitespace(node)
	{
		if (node.childNodes && node.childNodes.length)
			Array.prototype.slice.call(node.childNodes).forEach(removeWhitespace);
		if ((node.nodeType === 3 || node.nodeType === 4) && /^[ \r\n\t]*$/.test(node.textContent))
			node.parentNode.removeChild(node);
	}

	function createRow(table, data)
	{
		return safeAjax({
			type: "POST",
			url: `/_api/${table}`,
			contentType: "application/json",
			data: JSON.stringify(data)
		});
	}
	this.CreateRow = createRow;

	function deleteRow(table, id)
	{
		return safeAjax({
			type: "DELETE",
			url: `/_api/${table}(${id})`,
			contentType: "application/json"
		});
	}
	this.DeleteRow = deleteRow;

	function safeAjax(ajaxOptions) 
	{
		var deferredAjax = $.Deferred();
		shell.getTokenDeferred()
			.done((token) => 
			{
				if (!ajaxOptions.headers) 
					$.extend(ajaxOptions, { headers: { "__RequestVerificationToken": token } });
				else 
					ajaxOptions.headers["__RequestVerificationToken"] = token;
				
				$.ajax(ajaxOptions)
					.done((data, textStatus, jqXHR) => validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve))
					.fail(deferredAjax.reject);
			})
			.fail(() => deferredAjax.rejectWith(this, arguments));

		return deferredAjax.promise();	
	}
	
	function toggleRow(button, id, table, idIsAccount)
	{
		button.disabled = true;

		button.classList.toggle("noab-switch-off");
		const enabled = button.classList.toggle("noab-switch-on");
		button.firstChild.classList.toggle("noab-switch-thumb-off");
		button.firstChild.classList.toggle("noab-switch-thumb-on");
		
		const itemID = button.getAttribute("itemid");

		if (enabled)
		{
			let data = {};
			switch (table)
			{
				case "noab_specialties":
				case "noab_interests":
					data["noab_person@odata.bind"] = `/pyl_persons(${id})`;
					data["noab_subject@odata.bind"] = `/noab_subjects(${itemID})`;
					break;
					
				case "noab_memberships":
					if (idIsAccount)
						data["noab_member_account@odata.bind"] = `/accounts(${id})`;
					else
						data["noab_person@odata.bind"] = `/pyl_persons(${id})`;
					data["noab_association@odata.bind"] = `/noab_associations(${itemID})`;
					break;
					
				case "noab_serviceses":
					if (idIsAccount)
						data["noab_provider_account@odata.bind"] = `/accounts(${id})`;
					else
						data["noab_person@odata.bind"] = `/pyl_persons(${id})`;
					data["noab_businessactivity@odata.bind"] = `/noab_businessactivitieses(${itemID})`;
					break;
					
				case "noab_event_subjects":
					data["noab_eventtemplate@odata.bind"] = `/cay_eventtemplates(${id})`;
					data["noab_subject@odata.bind"] = `/noab_subjects(${itemID})`;
					break;

				case "noab_software_usages":
					data["noab_account@odata.bind"] = `/accounts(${id})`;
					data["noab_software@odata.bind"] = `/noab_softwares(${itemID})`;
					break;
					
				case "noab_targetaudiences":
					data["noab_account@odata.bind"] = `/accounts(${id})`;
					data["noab_industry@odata.bind"] = `/noab_branchs(${itemID})`;
					break;
					
				case "noab_expertises":
					data["noab_expert_account@odata.bind"] = `/accounts(${id})`;
					data["noab_subject@odata.bind"] = `/noab_subjects(${itemID})`;
					break;

				default:
					console.warn(`Not implemented case for NOAB.ToggleRow`);
					return;
			}

			createRow(table, data)
				.done((res, status, xhr) => 
				{
					button.setAttribute("owneditemid", xhr.getResponseHeader("entityid"));
					button.disabled = false;
				})
				.fail((a, b, c) => { console.log(a); console.log(b); console.log(c); });
		}
		else 
		{
			deleteRow(table, button.getAttribute("owneditemid"))
				.then(() => button.disabled = false);
		}
	}
	this.ToggleRow = toggleRow;
	
	function loadMap()
	{
		map = L.map('map').setView([52.2129919, 5.2793703], 7);
		L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
			maxZoom: 19,
			attribution: '&copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>'
		}).addTo(map);

		markerIcon = L.icon({
			iconUrl: '/noab-marker-account.png',
			iconSize: [15, 23],
			iconAnchor: [7, 23],
			popupAnchor: [0, -31]
		});
	}
	this.LoadMap = loadMap;
	
	function addMarkerForAccount(account)
	{
		if (!account)
			return;

		const latitude = account.address1_latitude;
		const longitude = account.address1_longitude;
		if (latitude && longitude)
		{	
			var marker = L.marker([latitude, longitude], { icon: markerIcon }).addTo(map);
			marker.bindPopup(
				`<div class="noab-leaflet-popup-content">
					<h4>${account.name}</h4>
					<p>${account.street} ${account.housenumber}${account.housenumberextra ?? ""}<br>${account.postalcode} ${account.city}</p>
				</div>`
				,
				{ className: "noab-leaflet-popup-content" }
			);
			marker.openPopup();
		}
	}
	this.AddMarkerForAccount = addMarkerForAccount;
	
	function linkSubgridOfContactsToProfile(profileRelativeUrl)
	{
		// When a grid with employees is loaded
		$(".entity-grid.subgrid").on("loaded", function() 
		{
			// Go over all rows
			$(this).find('td[data-attribute="fullname"]').each(async function()
			{
				// Get the ID, which is a contact ID
				const id = $(this).parent()[0].getAttribute('data-id');

				// Create a hyperlink to the profile of that person
				const link = document.createElement("a");
				link.href = `${profileRelativeUrl}?relationid=${id}`;
				link.text = this.innerHTML;

				// Replace the current content with the hyperlink
				this.innerHTML = '';
				this.appendChild(link);
			});
		});
	}
	this.LinkSubgridOfContactsToProfile = linkSubgridOfContactsToProfile;
	
	function removeActionMenuFromList(listSelector)
	{
		$(listSelector).on("loaded", function() 
		{
			$(this).find('div.dropdown.action').each(function()
			{
				this.remove();
			});
		});
	}
	this.RemoveActionMenuFromList = removeActionMenuFromList;
	
	function addQueryParametersToWebLinkSet()
	{
		const parameters = (new URLSearchParams(window.location.search)).toString();
		$('div .weblinks.list-group').find('a').each(function() { this.href += "?" + parameters; });
	}
	this.AddQueryParametersToWebLinkSet = addQueryParametersToWebLinkSet;
	
}).call(NOAB);